package com.atguigu.spzx.manger.service.impl;

import com.alibaba.excel.EasyExcel;
import com.atguigu.exception.BunnyException;
import com.atguigu.spzx.manger.excel.CategoryExcelListener;
import com.atguigu.spzx.manger.mapper.CategoryMapper;
import com.atguigu.spzx.manger.service.CategoryService;
import com.atguigu.spzx.model.entity.product.Category;
import com.atguigu.spzx.model.vo.product.CategoryExcelVo;
import com.atguigu.spzx.model.vo.result.ResultCodeEnum;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class CategoryServiceImpl implements CategoryService {
    @Autowired
    private CategoryMapper categoryMapper;

    /**
     * 根据parentId获取下级节点
     *
     * @param parentId 父级ID
     * @return 分类实体类列表
     */
    @Override
    public List<Category> findByParentId(Long parentId) {
        // 根据分类id查询它下面的所有的子分类数据
        List<Category> categoryList = categoryMapper.selectByParentId(parentId);
        if (!CollectionUtils.isEmpty(categoryList)) {
            categoryList.forEach(item -> {
                // 查询该分类下子分类的数量
                int count = categoryMapper.countByParentId(item.getId());
                item.setHasChildren(count > 0);
            });
        }
        return categoryList;
    }

    /**
     * 导出数据
     *
     * @param response HttpServletResponse
     */
    @Override
    public void exportData(HttpServletResponse response) {
        // 设置响应结果类型
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        String filename = URLEncoder.encode("分类数据", StandardCharsets.UTF_8);
        response.setHeader("Content-disposition", "attachment;filename=" + filename + ".xlsx");
        // 查询数据库中的数据
        List<Category> categoryList = categoryMapper.selectAll();
        ArrayList<CategoryExcelVo> excelVoArrayList = new ArrayList<>();
        // 将从数据库中查询到的Category对象转换成CategoryExcelVo对象
        categoryList.forEach(category -> {
            CategoryExcelVo vo = new CategoryExcelVo();
            BeanUtils.copyProperties(category, vo, CategoryExcelVo.class);
            excelVoArrayList.add(vo);
        });

        try {
            // 写出数据到浏览器端
            EasyExcel.write(response.getOutputStream(), CategoryExcelVo.class).sheet("分类数据").doWrite(excelVoArrayList);
        } catch (IOException exception) {
            log.error("文件写入失败：{}", exception.getMessage());
            throw new BunnyException(ResultCodeEnum.WRITE_FILE_ERROR);
        }
    }

    /**
     * 导入功能
     *
     * @param file Excel文件
     */
    @Override
    public void importData(MultipartFile file) {
        // 创建监听器对象，传递mapper对象
        CategoryExcelListener<CategoryExcelVo> listener = new CategoryExcelListener<>(categoryMapper);
        // 调用read方法读取excel数据
        try {
            EasyExcel.read(file.getInputStream(), CategoryExcelVo.class, listener).sheet().doRead();
        } catch (IOException e) {
            throw new BunnyException(ResultCodeEnum.DATA_ERROR);
        }
    }
}
